import { e as MastraStorage, T as TABLE_NAMES, S as StorageColumn, f as StorageThreadType, g as MessageType, h as StorageGetMessagesArg, E as EvalRow, W as WorkflowRunState } from '../../base-xpk2ifTz.cjs';
import '../../base-BihKcqDY.cjs';
import 'ai';
import '../../types-BtMyV38I.cjs';
import 'sift';
import 'zod';
import 'json-schema';
import '../../di/index.cjs';
import '../../index-CquI0inB.cjs';
import 'stream';
import 'pino';
import '@opentelemetry/api';
import 'xstate';
import 'node:events';
import '../../vector/index.cjs';
import '../../vector/filter/index.cjs';
import '../../tts/index.cjs';
import '../../deployer/index.cjs';
import '../../bundler/index.cjs';
import 'hono';
import 'hono/cors';
import 'hono-openapi';
import '@opentelemetry/sdk-trace-base';

interface LibSQLConfig {
    url: string;
    authToken?: string;
}
declare class LibSQLStore extends MastraStorage {
    private client;
    constructor({ config }: {
        config: LibSQLConfig;
    });
    protected rewriteDbUrl(url: string): string;
    private getCreateTableSQL;
    createTable({ tableName, schema, }: {
        tableName: TABLE_NAMES;
        schema: Record<string, StorageColumn>;
    }): Promise<void>;
    clearTable({ tableName }: {
        tableName: TABLE_NAMES;
    }): Promise<void>;
    private prepareStatement;
    insert({ tableName, record }: {
        tableName: TABLE_NAMES;
        record: Record<string, any>;
    }): Promise<void>;
    batchInsert({ tableName, records }: {
        tableName: TABLE_NAMES;
        records: Record<string, any>[];
    }): Promise<void>;
    load<R>({ tableName, keys }: {
        tableName: TABLE_NAMES;
        keys: Record<string, string>;
    }): Promise<R | null>;
    getThreadById({ threadId }: {
        threadId: string;
    }): Promise<StorageThreadType | null>;
    getThreadsByResourceId({ resourceId }: {
        resourceId: string;
    }): Promise<StorageThreadType[]>;
    saveThread({ thread }: {
        thread: StorageThreadType;
    }): Promise<StorageThreadType>;
    updateThread({ id, title, metadata, }: {
        id: string;
        title: string;
        metadata: Record<string, unknown>;
    }): Promise<StorageThreadType>;
    deleteThread({ threadId }: {
        threadId: string;
    }): Promise<void>;
    private parseRow;
    getMessages<T extends MessageType[]>({ threadId, selectBy }: StorageGetMessagesArg): Promise<T>;
    saveMessages({ messages }: {
        messages: MessageType[];
    }): Promise<MessageType[]>;
    private transformEvalRow;
    getEvalsByAgentName(agentName: string, type?: 'test' | 'live'): Promise<EvalRow[]>;
    getTraces({ name, scope, page, perPage, attributes, filters, }?: {
        name?: string;
        scope?: string;
        page: number;
        perPage: number;
        attributes?: Record<string, string>;
        filters?: Record<string, any>;
    }): Promise<any[]>;
    getWorkflowRuns({ workflowName, fromDate, toDate, limit, offset, }?: {
        workflowName?: string;
        fromDate?: Date;
        toDate?: Date;
        limit?: number;
        offset?: number;
    }): Promise<{
        runs: Array<{
            workflowName: string;
            runId: string;
            snapshot: WorkflowRunState | string;
            createdAt: Date;
            updatedAt: Date;
        }>;
        total: number;
    }>;
}

export { LibSQLStore as DefaultStorage, type LibSQLConfig, LibSQLStore };
